/* Copyright (C) 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import java.util.Map;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;

import org.dive4elements.river.artifacts.model.minfo.BedParametersResult;

public class BedQualityDensityProcessor extends DefaultProcessor {

    private final static Logger logger =
            Logger.getLogger(BedQualityDensityProcessor.class);

    public static final String I18N_AXIS_LABEL_DEFAULT =
        "Dichte [t/m^3]";
    public static final String I18N_AXIS_LABEL =
        "chart.bedquality.yaxis.label.density";

    protected String yAxisLabel;
    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible) {
        CallContext context = generator.getCallContext();
        StyledXYSeries series = new StyledXYSeries(bundle.getFacetDescription(),
                theme);
        Object data = bundle.getData(context);
        Map<String, String> metaData = bundle.getFacet().getMetaData();
        series.putMetaData(metaData, bundle.getArtifact(), context);
        yAxisLabel = metaData.get("Y");
        String facetName = bundle.getFacetName();
        double [][] points;

        if (facetName.equals(FacetTypes.BED_QUALITY_SEDIMENT_DENSITY_TOPLAYER)) {
            points = ((BedParametersResult) data).getDensityCapData();
        } else if (facetName.equals(FacetTypes.BED_QUALITY_SEDIMENT_DENSITY_SUBLAYER)) {
            points = ((BedParametersResult) data).getDensitySubData();
        } else {
            logger.error("Unknown facet name: " + facetName);
            return;
        }
        StyledSeriesBuilder.addPoints(series, points, true);

        generator.addAxisSeries(series, axisName, visible);
    }

    @Override
    public boolean canHandle(String facettype) {
        return facettype.equals(FacetTypes.BED_QUALITY_SEDIMENT_DENSITY_SUBLAYER) ||
            facettype.equals(FacetTypes.BED_QUALITY_SEDIMENT_DENSITY_TOPLAYER);
    }

    @Override
    public String getAxisLabel(DiagramGenerator generator) {
        if (yAxisLabel != null && !yAxisLabel.isEmpty()) {
            return generator.msg(yAxisLabel, I18N_AXIS_LABEL_DEFAULT);
        }
        return generator.msg(
                I18N_AXIS_LABEL,
                I18N_AXIS_LABEL_DEFAULT);
    }
}
